#pragma once

#include <string>
#include <queue>
#include <vector>
#include <thread>
#include <mutex>
#include <condition_variable>
#include <functional>
#include <optional>
#include <exception>
#include <memory>

#include "loggable.h"

class TaskManager : public Loggable {
public:
    using Error = std::optional<const std::exception*>;
    using Job = std::function<void ()>;
    using Result = std::function<void (Error)>;
    using Task = std::pair<Job, std::optional<Result>>;

    TaskManager(const std::shared_ptr<Logger>& logger);
    ~TaskManager();

    void start();
    void stop();
    void wait() const;
    void queue(const Job& job);
    void queue(const Job& job, const Result& result);
    bool busy() const;

private:
    void loop();
    void executeTask(const Task& task) const;

private:
    bool running;
    bool stopping;
    uint32_t maxThreads;
    uint32_t activeThreads;
    std::queue<Task> tasks;
    mutable std::mutex mutex;
    mutable std::condition_variable loopConditional;
    mutable std::condition_variable waitConditional;
    std::vector<std::thread> threads;
};
